import argparse

import argcomplete
from config import __version__, config
import aws_sessions_switcher
from aws_sessions_switcher import AwsAssume, util, config_collector, config_parser_util


def configure_environments_parser(env_parser):
    env_parser.add_argument(config.SHORT_VAR_PROJECT_NAME, config.LONG_VAR_PROJECT_NAME,
                            help="Name of the project", required=False)
    env_operations_sub_parser = env_parser.add_subparsers(title="environment-operations",
                                                          dest=config.SUB_ACTION)
    env_add_operation = env_operations_sub_parser.add_parser(config.sub_commands['ADD'])
    env_add_operation.add_argument(config.SHORT_VAR_PROJECT_NAME,
                                   config.LONG_VAR_PROJECT_NAME,
                                   help="Name of the project in which an environment needs to be added",
                                   required=True)
    env_delete_operation = env_operations_sub_parser.add_parser(config.sub_commands['DEL'])
    env_delete_operation.add_argument(config.SHORT_VAR_PROJECT_NAME,
                                      config.LONG_VAR_PROJECT_NAME,
                                      help="Name of the project in which an enviroment needs to be deleted",
                                      required=True)

    env_delete_operation.add_argument(config.SHORT_VAR_ENV_NAME,
                                      config.LONG_VAR_ENV_NAME,
                                      help="Name of the project in which an enviroment needs to be deleted",
                                      required=True)


class ArgumentParser:
    configured_projects = []
    configured_projects_parsers = {}

    def __init__(self) -> object:
        # MAIN Parser
        self.parent_parser = argparse.ArgumentParser(description=util.green_text("aws-sessions-switcher CLI"),
                                                     add_help=True)
        self.parent_parser.add_argument("-l", "--list",
                                        action='store_true',
                                        dest=config.ACTION_LIST_ASSUMPTIONS,
                                        help="Lists all the role assumptions that you can make"
                                        )

        self.parent_parser.add_argument("-v", "--version",
                                        action='version',
                                        version=util.green_text(f'aws-sessions-switcher {__version__.get_version()}'),
                                        help="Lists all the role assumptions that you can make"
                                        )

        self.sub_parser = self.parent_parser.add_subparsers(title="sub-command",
                                                            description="sub-command",
                                                            dest=config.ACTION,
                                                            required=False)

        # COMMAND => configure
        self.sub_parser.add_parser(config.sub_commands['CFGR'])

        # COMMAND => projects
        self.project_parser = self.sub_parser.add_parser(config.sub_commands['PRJ'])
        project_operations_subparser = self.project_parser.add_subparsers(title="project-operations",
                                                                          dest=config.SUB_ACTION)
        project_operations_subparser.add_parser(config.sub_commands['ADD'])
        delete_operation = project_operations_subparser.add_parser(config.sub_commands['DEL'])
        delete_operation.add_argument(config.SHORT_VAR_PROJECT_NAME,
                                      config.LONG_VAR_PROJECT_NAME,
                                      help="Name of the project to be deleted",
                                      required=True)

        # COMMAND => sessions
        self.sessions_parser = self.sub_parser.add_parser(config.sub_commands['SESSIONS'])
        self.sessions_sub_parser = self.sessions_parser.add_subparsers(title="session-operations",
                                                                       dest=config.SUB_ACTION)
        self.sessions_sub_parser.add_parser(config.sub_commands['SWT'])

        # COMMAND => environments
        self.environments_parser = self.sub_parser.add_parser(config.sub_commands['ENV'])
        configure_environments_parser(self.environments_parser)
        # COMMAND => env (same as the previous one, but short version)
        self.environments_parser_short = self.sub_parser.add_parser(config.sub_commands['SHORT_ENV'])
        configure_environments_parser(self.environments_parser_short)

        # COMMAND => reset (deletes all saved config)
        self.sub_parser.add_parser(config.sub_commands['RST'])

        # ENABLE AUTO COMPLETION OF COMMANDS
        argcomplete.autocomplete(self.parent_parser)

        # Instantiate the `aws-sessions-switcher` tool
        self.aws_assume = AwsAssume()

        # DYNAMIC => Now load already configured projects into the parser
        for project_name in self.aws_assume.list_projects(printable=False, validate_configuration=False):
            self.configured_projects.append(project_name)
            self.configured_projects_parsers[project_name] = self.sub_parser.add_parser(project_name)
            _sub = self.configured_projects_parsers[project_name].add_subparsers(title='project_environment',
                                                                                 dest='project_environment',
                                                                                 required=True)
            _envs = self.aws_assume.list_project_environments(project_name, printable=False)
            for environment in _envs:
                env_parser = _sub.add_parser(environment)
                role_parser = env_parser.add_subparsers(title="iam_role", dest='iam_role', required=True)
                roles = self.aws_assume.list_roles(project_name, environment, printable=False)
                {r: role_parser.add_parser(r) for r in roles}

        self.args = self.parent_parser.parse_args().__dict__

    def handle_projects_sub_operations(self):
        if self.args.get(config.SUB_ACTION) in [config.ACTION_LIST, None]:
            self.aws_assume.list_projects()
        elif self.args.get(config.SUB_ACTION) == config.ACTION_ADD:
            self.aws_assume.add_project()
        elif self.args.get(config.SUB_ACTION) == config.ACTION_DELETE:
            _ans = config_collector.ConfirmationDialog(
                f'Are you sure you want to delete project: [{self.args.get(config.VAR_PROJECT_NAME)}]?').get_answer()
            if _ans:
                self.aws_assume.delete_project(self.args.get(config.VAR_PROJECT_NAME))
        else:
            self.project_parser.error("No action requested, add -process or -upload'")

    def handle_environments_sub_operations(self, sub_action):
        if sub_action:
            if sub_action == config.ACTION_ADD:
                # TODO: ADD A NEW ENVIRONMENT
                print(util.yellow_text('Not supported currently. It will be available in later versions...'))
            elif sub_action == config.ACTION_DELETE:
                # TODO: DELETE AN ENVIRONMENT
                print(util.yellow_text('Not supported currently. It will be available in later versions...'))
        else:
            if self.args.get(config.VAR_PROJECT_NAME):
                self.aws_assume.list_project_environments(self.args.get(config.VAR_PROJECT_NAME))
            else:
                self.aws_assume.list_all_environments()

    def run(self):
        # print(util.yellow_text(self.args))
        action = self.args.get(config.ACTION)
        sub_action = self.args.get(config.SUB_ACTION)

        # Configure for initial run
        if action == config.ACTION_CONFIGURE:
            aws_sessions_switcher.configure()
            return

        # LIST ASSUMPTIONS THAT CAN BE MADE
        if self.args.get(config.ACTION_LIST_ASSUMPTIONS) or action is None:
            aws_sessions_switcher.validate_config_file()
            all_config = self.aws_assume.get_all_projects_config()
            for _cfg, _details in list(all_config.items()):
                _details['command'] = "aws-sessions-switcher {} {} {}".format(
                    _details['project_name'], _details['project_environment'], _details['role_name'])

            headers = ['assumptions', 'command_to_run']
            rows = [util.get_assumption_row(details) for project, details in
                    all_config.items()]
            util.print_table(headers, rows)

        # PROJECTS
        elif action == config.ACTION_PROJECT:
            self.handle_projects_sub_operations()

        # ENVIRONMENTS
        elif action in config.ACTION_ENVIRONMENT:
            self.handle_environments_sub_operations(sub_action)

        # SESSIONS
        elif action == config.ACTION_SESSIONS:
            # TODO: PROJECT NAME is Currently not used
            if sub_action == config.ACTION_SWITCH:
                sessions = self.aws_assume.get_active_sessions(self.args.get(config.VAR_PROJECT_NAME), printable=False)
                selected_session = config_collector.SelectionMenu(choices=sessions).get_answer()
                config_parser_util.switch_to_session(selected_session)
            else:
                self.aws_assume.get_active_sessions(self.args.get(config.VAR_PROJECT_NAME))
        elif action == config.ACTION_RESET:
            aws_sessions_switcher.perform_reset()
        # SESSIONS: Switcher: TODO; Switch between multiple sessions

        # TODO: Bash Autocompletion

        # ASSUME
        elif action in self.configured_projects:
            self.aws_assume.assume_role(project_name=action, environment=self.args['project_environment'],
                                        role=self.args['iam_role'])


def main():
    ArgumentParser().run()



import sys

import boto3
from botocore.exceptions import ClientError, ParamValidationError

from config import config
from aws_sessions_switcher import util


def get_sts_credentials(project_name, project_config, mfa_token, session_name):
    try:
        base_creds = config.get_base_credentials_for_project(project_name)
        sts_client = boto3.client(
            'sts',
            aws_access_key_id=base_creds['key_id'],
            aws_secret_access_key=base_creds['access_key']
        )

        response = sts_client.assume_role(
            RoleArn=project_config['role_arn'],
            RoleSessionName=session_name,
            DurationSeconds=int(project_config['mfa_device_session_duration']),
            SerialNumber=project_config['mfa_device_arn'],
            TokenCode=mfa_token
        )
        util.debug_log(f"Response from STS service: {response}")
        return response
    except ClientError as e:
        print(util.red_text("An error occured while calling "
                            "assume role: {}".format(e)))
        sys.exit(1)
    except ParamValidationError:
        e = sys.exc_info()[1]
        print(util.red_text("ERROR: " + e.args[0]))
        sys.exit(1)


def get_sts_credentials_without_mfa(project_name, project_config, session_name):
    try:
        base_creds = config.get_base_credentials_for_project(project_name)
        sts_client = boto3.client(
            'sts',
            aws_access_key_id=base_creds['key_id'],
            aws_secret_access_key=base_creds['access_key']
        )

        response = sts_client.assume_role(
            RoleArn=project_config['role_arn'],
            RoleSessionName=session_name,
            DurationSeconds=int(project_config['mfa_device_session_duration']),
        )
        util.debug_log(f"Response from STS service: {response}")
        return response
    except ClientError as e:
        print(util.red_text("An error occured while calling "
                            "assume role: {}".format(e)))
        sys.exit(1)
    except ParamValidationError:
        e = sys.exc_info()[1]
        print(util.red_text("ERROR: " + e.args[0]))
        sys.exit(1)



from __future__ import print_function, unicode_literals

from PyInquirer import style_from_dict, Token, prompt
from dataclasses import dataclass

from aws_sessions_switcher import util

default_style = style_from_dict({
    Token.Separator: '#cc5454',
    Token.QuestionMark: '#E91E63 bold',
    Token.Selected: '#673AB7 bold',
    Token.Pointer: '#673ab7 bold',
    Token.Instruction: '#2196f3',  # default
    Token.Answer: '#2196f3 bold',
    Token.Question: '#9ACD32',
})

selection_style = style_from_dict({
    Token.Separator: '#cc5454',
    Token.QuestionMark: '#2196f3 bold',
    Token.Selected: '#FFFF00 bold',
    Token.Pointer: '#673ab7 bold',
    Token.Instruction: '#2196f3',  # default
    Token.Answer: '#2196f3 bold',
    Token.Question: '#9ACD32',
})


class ConfigCollector:
    questions = [
        {
            'type': 'input',
            'name': 'project_name',
            'message': 'What\'s the project name?',
            'validate': util.generic_text_validator
        },
        {
            'type': 'input',
            'name': 'project_environment',
            'message': 'Type the environment identifier?',
            'validate': util.generic_text_validator
        },
        {
            'type': 'input',
            'name': 'role_arn',
            'message': 'Type the ARN of the AWS Role, you want to assume?',
        },
        {
            'type': 'input',
            'name': 'role_name',
            'message': 'Give a name to this role:',
            'validate': util.generic_text_validator
        },
        {
            'type': 'confirm',
            'name': 'mfa_required',
            'message': 'Is MFA Required?',
            'default': False,
        },
        {
            'type': 'input',
            'name': 'mfa_device_arn',
            'message': 'Type the ARN of the MFA device?',
            'when': lambda answers: answers['mfa_required'],
            'validate': util.not_empty
        },
        {
            'type': 'input',
            'name': 'mfa_device_session_duration',
            'message': 'Session duration in seconds? (Default: 3600)',
            'default': '3600',
            'when': lambda answers: answers['mfa_required'],
            'validate': util.numbers_only
        },
    ]

    def collect(self):
        answers = prompt(self.questions, style=default_style)
        # pprint(answers)
        return answers


@dataclass
class ConfirmationDialog:
    question: str = None

    def get_answer(self):
        answers = prompt([{
            'type': 'confirm',
            'name': 'confirmation',
            'message': self.question,
            'default': False,
        }], style=default_style)
        return answers.get('confirmation', 'N')


@dataclass
class InputDialog:
    question: str = None

    def get_answer(self):
        answers = prompt([{
            'type': 'input',
            'name': 'value',
            'message': self.question,
            'default': '',
            'validate': util.numbers_only
        }], style=default_style)
        return answers.get('value', '')


@dataclass
class SelectionMenu:
    choices: list = None

    def prepare_list(self):
        return [dict({'name': x}) for x in self.choices]

    def get_answer(self):
        answers = prompt(questions=[{
            'type': 'list',
            'qmark': '❯',
            'message': 'Select a session to switch to',
            'name': 'switch_to_session',
            'choices': self.prepare_list(),
            'validate': lambda answer: 'You must choose at least one topping.' \
                if len(answer) == 0 else True
        }], style=selection_style)

        return answers.get('switch_to_session', None)



import copy
import os
import sys

import configparser

from config import config
from aws_sessions_switcher import util, config_parser_util, config_collector, aws_client


def validate_config_file():
    if not os.path.isfile(config.AWS_ASSUME_CONFIG_PATH):
        util.error_log(f'Could not locate configuration file at "{config.AWS_ASSUME_CONFIG_PATH}"', _exit=False)
        util.info_log(f"Run `aws-sessions-switcher configure` to create one")
        sys.exit(1)


def configure(write_mode='w', check_file_existence=True):
    if check_file_existence and os.path.isfile(config.AWS_ASSUME_CONFIG_PATH):
        util.error_log(
            'File %s already exists.'
            'Run `aws-sessions-switcher projects add` if you want to add a new project configuration.'
            'Type \'aws-sessions-switcher -h\' to see all the available sub-commands' % (
                config.AWS_ASSUME_CONFIG_PATH,))
    else:
        _collector = config_collector.ConfigCollector()
        answers = _collector.collect()
        if not answers:
            return
        cfg_parser = configparser.ConfigParser()
        cfg_parser[f"{answers['project_name']}-{answers['project_environment']}"] = answers
        with open(config.AWS_ASSUME_CONFIG_PATH, write_mode) as configfile:
            cfg_parser.write(configfile)

        print(util.yellow_text(
            f"Note: Make sure to put your security credentials under "
            f"\"{util.get_base_aws_profile_for_project(answers['project_name'])}\" "
            f"section of your AWS Credentials"))


def perform_reset():
    if os.path.exists(config.AWS_ASSUME_CONFIG_PATH):
        try:
            ans = config_collector.ConfirmationDialog(
                f'This file => "{config.AWS_ASSUME_CONFIG_PATH}" will be deleted. '
                f'Are you sure you want to perform a reset?: ').get_answer()
            if ans:
                os.remove(config.AWS_ASSUME_CONFIG_PATH)
                util.info_log(f'The file "{config.AWS_ASSUME_CONFIG_PATH}" is deleted')
        except Exception:
            util.error_log(f'The file "{config.AWS_ASSUME_CONFIG_PATH}" could not be deleted')
    else:
        util.error_log(f'The file "{config.AWS_ASSUME_CONFIG_PATH}" does not exist')
    pass


class AwsAssume:
    all_projects_config = {}
    all_active_sessions = config_parser_util.get_all_active_sessions()

    def __init__(self, action=None) -> None:
        if os.path.isfile(config.AWS_ASSUME_CONFIG_PATH):
            self.all_projects_config = config_parser_util.get_all_projects()

    def get_all_projects_config(self):
        return self.all_projects_config

    def list_projects(self, printable=True, validate_configuration=True) -> []:
        if validate_configuration:
            validate_config_file()

        _pjts = []
        for env, details in self.all_projects_config.items():
            if details.get('project_name') not in _pjts:
                if printable:
                    print(util.green_text(f"- {details['project_name']}"))
                _pjts.append(details['project_name'])
        return _pjts

    def list_project_environments(self, project_name, printable=True):
        validate_config_file()
        _pjt_envs = []
        for env, details in self.all_projects_config.items():
            if details.get('project_name') and project_name == details['project_name']:
                _pjt_envs.append(details['project_environment'])
                if printable:
                    print(util.green_text(f"- {details['project_environment']}"))
        return _pjt_envs

    def list_all_environments(self, printable=True, with_project_prefix=True):
        validate_config_file()
        for env, details in self.all_projects_config.items():
            prefix = f"{details['project_name']}-" if with_project_prefix else ''
            if printable:
                print(util.green_text(f"- {prefix}{details['project_environment']}"))

    def list_roles(self, project_name, environment, printable=False) -> []:
        validate_config_file()
        roles = []
        for env, details in self.all_projects_config.items():
            if details.get('project_name') and project_name == details['project_name'] and environment == details[
                'project_environment']:
                roles.append(details['role_name'])
                if printable:
                    util.info_log(f"{details['role_name']} => {details['role_arn']}")
        return roles

    def add_project(self):
        validate_config_file()
        configure('a', False)
        # new_project = config_collector.ConfigCollector().collect()
        # self.all_projects_config.update(new_project)
        # cfg_parser = configparser.ConfigParser()
        # cfg_parser[f"{new_project['project_name']}-{new_project['project_environment']}"] = new_project
        # with open(config.AWS_ASSUME_CONFIG_PATH, 'a') as configfile:
        #     cfg_parser.write(configfile)

    def delete_project(self, project_name):
        validate_config_file()
        configs = copy.deepcopy(self.all_projects_config)
        for cfg in configs:
            if project_name in cfg:
                self.all_projects_config.pop(cfg)

        writer = configparser.ConfigParser()
        for name, value in self.all_projects_config.items():
            writer[name] = value
        with open(config.AWS_ASSUME_CONFIG_PATH, 'w') as configfile:
            writer.write(configfile)

    def assume_role(self, project_name, environment, role):
        project_config = self.all_projects_config[f'{project_name}-{environment}']
        util.info_log(f"Attempting to assume role: \"{role}\" using ARN: \"{project_config['role_arn']}\" "
                      f"on project: {project_name}")
        session_name = f"session-{project_name}-{environment}"
        options = [
            ('aws_access_key_id', 'AccessKeyId'),
            ('aws_secret_access_key', 'SecretAccessKey'),
            ('aws_session_token', 'SessionToken'),
            ('aws_security_token', 'SessionToken'),
        ]

        if project_config['mfa_required'] == 'True':
            mfa_token = config_collector.InputDialog(
                f"MFA TOKEN for device {project_config['mfa_device_arn']}").get_answer()
            session_creds = aws_client.get_sts_credentials(project_name, project_config, mfa_token, session_name)
        else:
            session_creds = aws_client.get_sts_credentials_without_mfa(project_name, project_config, session_name)

        new_session = {k: session_creds['Credentials'][v] for k, v in options}
        new_session.update(
            {'expiration': session_creds['Credentials']['Expiration'].strftime(config.EXPIRATION_TIMESTAMP_FORMAT)})
        config_parser_util.replace_config_section(config.AWS_ASSUME_CONFIG_PATH, session_name, new_session)

        # replace the default profile in the AWS_CREDS file
        config_parser_util.replace_config_section(config.AWS_CREDS_PATH, 'default', new_session)
        print(util.green_text('- SUCCESS!'))


    def get_active_sessions(self, project_name, printable=True):
        validate_config_file()
        sessions = self.all_active_sessions
        remaining_times = {session_name: util.get_remaining_time(session_details['expiration']) for
                           session_name, session_details in sessions.items()}
        if not remaining_times:
            print(util.yellow_text(
                f'- No active sessions present. Run `aws-sessions-switcher -l` to see all possible role assumptions you can make'))
            return
        else:
            headers = ['session_name', 'remaining_time', 'configured_to_be_used_with_aws_command']
            rows = [util.get_session_row(session_name, remaining_time, sessions[session_name]) for
                    session_name, remaining_time in remaining_times.items()]
            if printable:
                util.print_table(headers, rows)
                print(
                    f'Note: If {util.yellow_text("`configured_to_be_used_with_aws_command`")}is False,\n'
                    f'run {util.green_text("`aws-sessions-switcher sessions switch`")} and select this session to activate it')
            return [session for session in remaining_times]



import os
import sys

import configparser
from configparser import NoOptionError, NoSectionError

from aws_sessions_switcher import util

AWS_CREDS_PATH = '%s/.aws/credentials' % (os.path.expanduser('~'),)
AWS_ASSUME_CONFIG_PATH = f'%s/.aws/{os.getenv("AWS_SESSIONS_SWITCHER_CONFIG_FILENAME", "sessions_switcher")}' % (
    os.path.expanduser('~'),)
AWS_ASSUME_BASE_CREDENTIALS_IDENTIFIER_PREFIX = 'aws-sessions-switcher-'
EXPIRATION_TIMESTAMP_FORMAT = '%Y-%m-%d %H:%M:%S'

# ACTIONS
ACTION = 'action'
SUB_ACTION = 'sub_action'
ACTION_LIST_ASSUMPTIONS = 'list_assumptions'
ACTION_ENVIRONMENT = ['env', 'environments']
ACTION_LIST = 'ls'
ACTION_ADD = 'add'
ACTION_DELETE = 'delete'
ACTION_CONFIGURE = 'configure'
ACTION_PROJECT = 'projects'
ACTION_RESET = 'reset'
ACTION_SESSIONS = 'sessions'
ACTION_SWITCH = 'switch'

# VARIABLES
VAR_PROJECT_NAME = 'project_name'
LONG_VAR_PROJECT_NAME = '--project-name'
SHORT_VAR_PROJECT_NAME = '-n'
SHORT_VAR_ENV_NAME = '-e'
LONG_VAR_ENV_NAME = '--env-name'

sub_commands = {
    'CFGR': ACTION_CONFIGURE,
    'PRJ': ACTION_PROJECT,
    'SESSIONS': ACTION_SESSIONS,
    'ENV': ACTION_ENVIRONMENT[0],
    'SHORT_ENV': ACTION_ENVIRONMENT[1],
    'LIST': ACTION_LIST,
    'ADD': ACTION_ADD,
    'DEL': ACTION_DELETE,
    'RST': ACTION_RESET,
    'SWT': ACTION_SWITCH,
}

project_sub_commands = {
    'LIST': ACTION_LIST,
    'ADD': ACTION_ADD,
    'DELETE': ACTION_DELETE,
}


def get_base_aws_config():
    config = configparser.RawConfigParser()
    try:
        config.read(AWS_CREDS_PATH)
    except configparser.ParsingError:
        e = sys.exc_info()[1]
        print(util.red_text("There was a problem reading or parsing "
                            "your credentials file: %s" % (e.args[0],)))
    return config


def get_base_credentials_for_project(project_name):
    base_config = get_base_aws_config()
    base_profile_name = util.get_base_aws_profile_for_project(project_name)
    try:
        base_credentials_section = base_config[base_profile_name]
        key_id = base_credentials_section['aws_access_key_id']
        access_key = base_credentials_section['aws_secret_access_key']
    except NoSectionError and KeyError:
        util.error_log(
            "Credentials for profile '[%s]' is missing. "
            "You must add this section to your AWS credentials file." % (base_profile_name,))
    except NoOptionError as e:
        util.error_log(e)

    return {
        'key_id': key_id,
        'access_key': access_key
    }



import logging
import re
import sys

import configparser
import datetime
import time
from prettytable import PrettyTable

from config import config


def setup_logger(logger, level=logging.DEBUG):
    stdout_handler = logging.StreamHandler(stream=sys.stdout)
    stdout_handler.setFormatter(
        logging.Formatter('%(levelname)s - %(message)s'))
    stdout_handler.setLevel(level)
    logger.addHandler(stdout_handler)
    logger.setLevel(level)


default_logger = logging.getLogger('aws_sessions_switcher')
setup_logger(default_logger, logging.INFO)


def debug_log(message, logger=default_logger, _nocolor=False):
    if _nocolor:
        logger.debug(message)
    else:
        logger.debug(yellow_text(message))


def info_log(message, logger=default_logger, _nocolor=False):
    if _nocolor:
        logger.info(message)
    else:
        logger.info(yellow_text(message))


def error_log(message, logger=default_logger, _exit=True, _nocolor=False):
    if _nocolor:
        logger.error(message)
    else:
        logger.error(red_text(message))
    if _exit:
        sys.exit(1)


#
# def prompter():
#     try:
#         console_input = raw_input
#     except NameError:
#         console_input = input
#
#     return console_input


def colored(r, g, b, text):
    return "\033[38;2;{};{};{}m{} \033[38;2;255;255;255m".format(r, g, b, text)


def blue_text(text):
    return colored(0, 0, 255, text)


def yellow_text(text):
    return colored(255, 255, 0, text)


def green_text(text):
    # return colored(0, 255, 0, text)
    return colored(154, 205, 50, text)


def red_text(text):
    return colored(255, 0, 0, text)


def generic_text_validator(project_name):
    err_msg = 'Only letters, numbers & underscores are allowed'
    return bool(re.match('^[a-zA-Z0-9_]+$', project_name)) or err_msg


def not_empty(val):
    err_msg = 'Cannot be empty'
    return val != "" or err_msg


def numbers_only(val):
    err_msg = 'Only numbers are allowed'
    return bool(re.match('^[0-9]+$', val)) or err_msg


def is_aws_session_active(session_name, session_details):
    try:
        cfg_parser_aws_creds_file = configparser.ConfigParser()
        cfg_parser_aws_creds_file.read(config.AWS_CREDS_PATH)
        if cfg_parser_aws_creds_file['default'] == session_details:
            return True
        return False
    except Exception:
        e = sys.exc_info()[1]
        print(red_text("Unable to check if session %s is active => %s" % (session_name, e.args[0])))


def get_session_time_difference_in_secs(expiration_time):
    exp_ts = time.mktime(datetime.datetime.strptime(expiration_time, config.EXPIRATION_TIMESTAMP_FORMAT).timetuple())
    cur_ts = time.mktime(datetime.datetime.now(datetime.timezone.utc).timetuple())
    return exp_ts - cur_ts


def get_remaining_time(expiration_time):
    _hrs, _min, _sec = str(datetime.timedelta(seconds=get_session_time_difference_in_secs(expiration_time))).split(":")
    return f'{_hrs}h : {_min}m : {_sec}s'


def is_session_expired(session_details):
    if get_session_time_difference_in_secs(session_details['expiration']) <= 0:
        return True
    return False


def get_assumption_row(project_details):
    return [
        f'{project_details["project_name"]}-{project_details["project_environment"]}-{project_details["role_name"]}',
        project_details['command']]


def print_table(headers, rows):
    t = PrettyTable([green_text(h) for h in headers])
    for row in rows:
        t.add_row([yellow_text(item) for item in row])
    print(t)


def get_base_aws_profile_for_project(project_name):
    return f'{config.AWS_ASSUME_BASE_CREDENTIALS_IDENTIFIER_PREFIX}{project_name}'


def get_session_row(session_name, remaining_time, session_details):
    return [session_name, remaining_time, is_aws_session_active(session_name, session_details)]



import configparser
import sys

from config import config
from aws_sessions_switcher import util


def replace_config_section(file_name, section_name, section_value):
    cfg_parser = configparser.ConfigParser()
    try:
        cfg_parser.read(file_name)
        cfg_parser[section_name] = section_value
        with open(file_name, 'w') as configfile:
            cfg_parser.write(configfile)
    except configparser.ParsingError:
        e = sys.exc_info()[1]
        print(util.red_text("There was a problem reading or parsing "
                            "file: %s" % (e.args[0],)))
        sys.exit(1)


def get_aws_creds_parser():
    cfg_parser = configparser.ConfigParser()
    cfg_parser.read(config.AWS_CREDS_PATH)
    return cfg_parser


def get_aws_assume_config_parser():
    cfg_parser = configparser.ConfigParser()
    cfg_parser.read(config.AWS_ASSUME_CONFIG_PATH)
    return cfg_parser


def replace_aws_assume_config(cfg_parser_with_new_config):
    with open(config.AWS_ASSUME_CONFIG_PATH, 'w') as configfile:
        cfg_parser_with_new_config.write(configfile)


def get_all_projects():
    cfg_parser = get_aws_assume_config_parser()
    # Read only the project configuration, excluding the sessions
    return {s: dict(cfg_parser.items(s)) for s in cfg_parser.sections() if
            not s.startswith('session-')}


def get_all_active_sessions():
    cfg_parser = get_aws_assume_config_parser()
    all_sessions = {s: dict(cfg_parser.items(s)) for s in cfg_parser.sections() if s.startswith('session-')}
    for session, session_details in list(all_sessions.items()):
        if util.is_session_expired(session_details):
            del all_sessions[session]
            cfg_parser.remove_section(session)
            replace_aws_assume_config(cfg_parser)
    return all_sessions


def switch_to_session(session_name):
    if session_name:
        all_sessions = get_all_active_sessions()
        if session_name not in all_sessions:
            util.error_log(f'Session {session_name} unavailable')

        # replace the default profile in the AWS_CREDS file
        replace_config_section(config.AWS_CREDS_PATH, 'default', all_sessions[session_name])
        print('INFO: Switched to => ' + util.green_text('{}'.format(session_name)))


def is_session_usable_in_aws_creds(session_details):
    aws_creds_config = get_aws_creds_parser()
    aws_creds_sections = aws_creds_config.sections()
    if 'default' in aws_creds_sections:
        print('EQUALITY: %s' % (aws_creds_sections['default'] == session_details))
        return aws_creds_sections['default'] == session_details
    return False



import setuptools
from config import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aws_sessions_switcher",
    version=__version__.get_version(),
    author="Balan Jayavictor",
    author_email="dev.balan.jayavictor@gmail.com",
    description="A tool to help switching between multiple AWS environments easy and seamless",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DEV-BALAN-JAYAVICTOR/aws-sessions-switcher",
    packages=['aws_sessions_switcher', 'config'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
    install_requires=[
        'configparser>=5.0.0',
        'boto3>=1.13.24',
        'pyinquirer>=1.0.3,<2',
        'dataclasses>=0.6,<1',
        'argcomplete>=1.11.1,<2',
        'prettytable>=0.7.2,<1'
    ],
    entry_points={
        'console_scripts': [
            'aws-sessions-switcher=aws_sessions_switcher.argument_parser:main',
        ],
    },
)
